from zope.interface import implementer
from zope import component
from ceo_common.interfaces import IDatabaseService, IConfig
from ceo_common.errors import DatabaseConnectionError, DatabasePermissionError
from ceod.utils import gen_password
from ceod.db.utils import response_is_empty
from mysql.connector import connect
from mysql.connector.errors import InterfaceError, ProgrammingError


@implementer(IDatabaseService)
class MySQLService:
    def __init__(self):
        self.type = 'mysql'
        config = component.getUtility(IConfig)
        self.auth_username = config.get('mysql_username')
        self.auth_password = config.get('mysql_password')

    def create_db(self, username: str) -> str:
        password = gen_password()
        search_for_user = f"SELECT user FROM mysql.user WHERE user='{username}'"
        search_for_db = f"SHOW DATABASES LIKE '{username}'"
        create_user = f"CREATE USER '{username}'@'localhost' IDENTIFIED BY %(password)s"
        reset_password = f"ALTER USER '{username}'@'localhost' IDENTIFIED BY %(password)s"
        create_database = f"""
        CREATE DATABASE {username};
        GRANT ALL PRIVILEGES ON {username}.* TO '{username}'@'localhost';
        """
        try:
            with connect(
                host='localhost',
                user=self.auth_username,
                password=self.auth_password,
            ) as con:
                with con.cursor() as cursor:
                    if response_is_empty(search_for_user, con):
                        cursor.execute(create_user, {'password': password})
                    else:
                        cursor.execute(reset_password, {'password': password})
                    if response_is_empty(search_for_db, con):
                        cursor.execute(create_database)
            return password
        except InterfaceError:
            raise DatabaseConnectionError()
        except ProgrammingError:
            raise DatabasePermissionError()

    def delete_db(self, username: str):
        drop_user = f"DROP USER IF EXISTS '{username}'@'localhost'"
        drop_db = f"DROP DATABASE IF EXISTS {username}"
        try:
            with connect(
                    host='localhost',
                    user=self.auth_username,
                    password=self.auth_password,
            ) as con:
                with con.cursor() as cursor:
                    cursor.execute(drop_user)
                    cursor.execute(drop_db)
        except InterfaceError:
            raise DatabaseConnectionError()
        except ProgrammingError:
            raise DatabasePermissionError()
